/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var RemovalMethods = {
    ALL: 'all',
    SINGULAR_LAST: 'singular-last',
    SINGULAR_FIRST: 'singular-first',
    AT_INDEX: 'at-index',
    RANGE: 'range',
};

var noop = function () { return void 0; };
var isFunction = function (item) { return typeof item === 'function'; };
var isNumber = function (item) {
    return typeof item === 'number' && !isNaN(item);
};
var isString = function (item) { return typeof item === 'string'; };
var isArray = function (item) { return item.constructor === Array; };
var isEmptyString = function (item) { return isString(item) && !item; };
var assignFunction = function (item) {
    if (isFunction(item)) {
        return item;
    }
};
var assignNumber = function (number, defaultNumber, listLength) {
    if (isNumber(number)) {
        if (number === 0 || number === Infinity) {
            return listLength;
        }
        return number;
    }
    return defaultNumber;
};

function DefaultOptions(name) {
    return {
        concurrency: 3,
        autoStart: false,
        name: name,
        onAdd: noop,
        onStart: noop,
        onRun: noop,
        onDone: noop,
        onEnd: noop,
        onRemove: noop,
    };
}

/**
 * Handles running a task, and updating metadata.
 */
function run() {
    var _a, _b, _c;
    if (this.tasks.list.length) {
        if (this.tasks.running < this.options.concurrency) {
            var task = this.tasks.list.shift();
            if (task) {
                this.tasks.running++;
                if (isFunction(this.options.onRun)) {
                    var _d = this, tasks = _d.tasks, duration = _d.duration;
                    this.options.onRun({ task: task, tasks: tasks, duration: duration });
                }
                this.strategy(task, done.bind(this));
            }
        }
    }
    else {
        if (this.tasks.completed === this.tasks.total) {
            this.duration.end = new Date();
            this.duration.total =
                ((_a = this.duration.end.valueOf()) !== null && _a !== void 0 ? _a : 0) -
                    ((_c = (_b = this.duration.start) === null || _b === void 0 ? void 0 : _b.valueOf()) !== null && _c !== void 0 ? _c : 0);
            this.setWorking(false);
            if (isFunction(this.options.onEnd)) {
                var _e = this, tasks = _e.tasks, duration = _e.duration;
                this.options.onEnd({ tasks: tasks, duration: duration });
            }
        }
    }
}
/**
 * Checks if slots are available and fills them up with tasks in the queue. Calls `run` internally.
 */
function runPending() {
    if (this.tasks.running < this.options.concurrency) {
        for (var i = this.tasks.running; i < this.options.concurrency; i++) {
            run.call(this);
        }
    }
}
/**
 * The done function marks a task as done, and frees a slot in the queue, subsequently calling `run` to fill that slot up.
 * @param result - The result of the task which can be accessed in `onDone`.
 */
var done = function done(result) {
    var _a, _b;
    this.tasks.completed++;
    this.tasks.running--;
    this.duration.total = Date.now() - ((_b = (_a = this.duration.start) === null || _a === void 0 ? void 0 : _a.valueOf()) !== null && _b !== void 0 ? _b : 0);
    if (isFunction(this.options.onDone)) {
        var tasks = this.tasks;
        this.options.onDone({ tasks: tasks, result: result });
    }
    if (!this.__destroyed) {
        run.call(this);
    }
};
/**
 * Checks whether the runner is free to start and triggers `onStart` method (if provided).
 */
function startCheck() {
    if (!this.__working) {
        this.duration.start = new Date();
        this.setWorking(true);
        if (isFunction(this.options.onStart)) {
            var _a = this, tasks = _a.tasks, duration = _a.duration;
            this.options.onStart({ tasks: tasks, duration: duration });
        }
    }
}
/**
 * Triggers the `onAdd` method (if provided).
 */
function addCheck() {
    if (isFunction(this.options.onAdd)) {
        var tasks = this.tasks;
        this.options.onAdd({ tasks: tasks });
    }
}
/**
 * Triggers the `onRemove` method (if provided).
 */
function removeCheck(method, removedTasks) {
    if (isFunction(this.options.onRemove)) {
        var tasks = this.tasks;
        this.options.onRemove({ tasks: tasks, method: method, removedTasks: removedTasks });
    }
}
/**
 * Checks whether the runner is free to start and runs pending. Triggers both `onStart` and `onRun` methods (if provided).
 */
function startCheckAndRunPending() {
    startCheck.call(this);
    runPending.call(this);
}

var CoreRunner = /** @class */ (function () {
    function CoreRunner(strategy, options) {
        this.__working = false;
        this.__destroyed = false;
        this.__paused = false;
        this.tasks = {
            total: 0,
            completed: 0,
            running: 0,
            list: [],
        };
        this.duration = {
            total: 0,
        };
        this.options = __assign(__assign({}, DefaultOptions("Runner-" + ++CoreRunner.runnerCount)), options);
        this.strategy = strategy;
        Object.seal(this);
    }
    CoreRunner.prototype.isBusy = function () {
        return this.__working;
    };
    CoreRunner.prototype.setWorking = function (working) {
        this.__working = working;
    };
    CoreRunner.prototype.setDestroyed = function (destroyed) {
        this.__destroyed = destroyed;
    };
    CoreRunner.prototype.start = function () {
        if (this.__working || this.__destroyed) {
            return false;
        }
        if (this.options.autoStart) {
            return false;
        }
        startCheck.call(this);
        runPending.call(this);
        return true;
    };
    CoreRunner.prototype.destroy = function () {
        this.__destroyed = true;
        this.__working = false;
        return true;
    };
    CoreRunner.prototype.on = function (event, callback) {
        var newEvent = "on" + (event.charAt(0).toUpperCase() + event.substr(1));
        this.options[newEvent] = callback;
    };
    CoreRunner.prototype.off = function (event) {
        var newEvent = "on" + (event.charAt(0).toUpperCase() + event.substr(1));
        this.options[newEvent] = noop;
    };
    CoreRunner.prototype.add = function (task, first) {
        if (first === void 0) { first = false; }
        if (first) {
            this.tasks.list.unshift(task);
        }
        else {
            this.tasks.list.push(task);
        }
        if (this.options.autoStart) {
            startCheckAndRunPending.call(this);
        }
        addCheck.call(this);
        return ++this.tasks.total;
    };
    CoreRunner.prototype.addFirst = function (task) {
        return this.add(task, true);
    };
    CoreRunner.prototype.addMultiple = function (tasks, first) {
        if (first === void 0) { first = false; }
        if (first) {
            this.tasks.list = __spreadArrays(tasks, this.tasks.list);
        }
        else {
            this.tasks.list = __spreadArrays(this.tasks.list, tasks);
        }
        if (this.options.autoStart) {
            startCheckAndRunPending.call(this);
        }
        addCheck.call(this);
        return (this.tasks.total =
            this.tasks.list.length + this.tasks.completed);
    };
    CoreRunner.prototype.addMultipleFirst = function (tasks) {
        return this.addMultiple(tasks, true);
    };
    CoreRunner.prototype.remove = function (first) {
        if (first === void 0) { first = false; }
        var task;
        if (first) {
            task = this.tasks.list.shift();
        }
        else {
            task = this.tasks.list.pop();
        }
        --this.tasks.total;
        removeCheck.call(this, first
            ? RemovalMethods.SINGULAR_FIRST
            : RemovalMethods.SINGULAR_LAST, task ? [task] : []);
        return task;
    };
    CoreRunner.prototype.removeFirst = function () {
        var task = this.remove(true);
        removeCheck.call(this, RemovalMethods.SINGULAR_FIRST, task ? [task] : []);
        return task;
    };
    CoreRunner.prototype.removeAt = function (index) {
        var tasks = this.tasks.list.splice(index, 1);
        --this.tasks.total;
        removeCheck.call(this, RemovalMethods.AT_INDEX, tasks);
        return tasks[0];
    };
    CoreRunner.prototype.removeRange = function (startIndex, endIndex) {
        var tasks = this.tasks.list.splice(startIndex, endIndex + 1);
        this.tasks.total -= tasks.length;
        removeCheck.call(this, RemovalMethods.RANGE, tasks);
        return tasks;
    };
    CoreRunner.prototype.removeAll = function () {
        var tasks = this.tasks.list.slice();
        this.tasks.list = [];
        this.tasks.total = this.tasks.completed;
        removeCheck.call(this, RemovalMethods.ALL, tasks);
        return tasks;
    };
    CoreRunner.runnerCount = 0;
    return CoreRunner;
}());

var Strategy = function (task, done) {
    task(done);
    process.on('unhandledRejection', function (e) {
        throw e;
    });
};

var TaskRunner = /** @class */ (function (_super) {
    __extends(TaskRunner, _super);
    function TaskRunner(options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        var strategy = options.strategy, otherOptions = __rest(options, ["strategy"]);
        _this = _super.call(this, strategy || Strategy, otherOptions) || this;
        return _this;
    }
    return TaskRunner;
}(CoreRunner));

export { CoreRunner, DefaultOptions, Strategy as DefaultStrategy, RemovalMethods, TaskRunner, assignFunction, assignNumber, isArray, isEmptyString, isFunction, isNumber, isString, noop };
//# sourceMappingURL=index.js.map
